@@ -1,4 +1,5 @@
 use bp7::dtntime::DtnTimeHelpers;
+use bp7::flags::BlockControlFlags;
 use bp7::helpers::*;
 use bp7::primary::PrimaryBlock;
 use bp7::*;
@@ -29,7 +30,7 @@ fn manifest_to_primary(manifest: &str) -> PrimaryBlock {
         .split('\n')
         .map(|f| f.trim())
         .filter(|l| !l.is_empty() || l.starts_with("^#"))
-        .filter(|l| l.contains("="))
+        .filter(|l| l.contains('='))
     {
         let result: Vec<&str> = line.splitn(2, '=').map(|f| f.trim()).collect();
         match result[0] {
@@ -57,7 +58,7 @@ fn manifest_to_primary(manifest: &str) -> PrimaryBlock {
     primary.build().expect("error building primary block")
 }
 fn generate_bundle(primary_block: PrimaryBlock, payload: Vec<u8>, hex: bool) {
-    let payload_block = bp7::new_payload_block(0, payload);
+    let payload_block = bp7::new_payload_block(BlockControlFlags::empty(), payload);
 
     let mut b = bundle::Bundle::new(primary_block, vec![payload_block]);
 
